/*****************************************************************************
 *                                                                           *
 *                Licensed Materials - Property of IBM Corp.                 *
 *                  5648-C60 (c) Copyright IBM Corp. 2000                    *
 *                                                                           *
 *       MQJMS Standalone Application Server Facilities (ASF) Samples        *
 *                                                                           *
 *                                                                           *
 *  FILENAME:     ASFClient5.java                                            *
 *                                                                           *
 *  DESCRIPTION:  Exercises the durable connection consumer functionality    *
 *                in MQJMS. This is an interactive application which allows  *
 *                users to create, reactivate and delete durable connection  *
 *                consumers.                                                 *
 *                                                                           *
 *                Everything is performed using a single ServerSessionPool   *
 *                and a single MessageListenerFactory.                       *
 *                                                                           *
 *                Further information about this class and its usage can be  *
 *                found in the 'Websphere MQ Using Java' documentation.      *
 *                                                                           *
 *  USAGE:        java ASFClient5 [options]                                  *
 *                                                                           *
 *                See documentation for details of the command-line syntax.  *
 *                                                                           *
 *  HISTORY:      010131 - First released as part of SupportPac MA88 v2      *
 *                                                                           *
 *****************************************************************************/

import javax.jms.*;
import javax.naming.*;
import java.util.Hashtable;
import java.io.*;

public class ASFClient5 {
   // These are the defaults for the parameters which can be over-ridden
   // on the command line

   // Will we use JNDI for administered object creation?
   private static boolean useJNDI    = true;
   // JNDI parameters
   private static String  tcfLookup  = "cn=tcf";
   private static String  tLookup    = "cn=t";
   private static String  icf        = "com.sun.jndi.ldap.LdapCtxFactory";
   private static String  url        = "ldap://localhost/o=ibm,c=us";
   // Non-JNDI parameters
   private static String  qMgrName   = "";
   private static String  tName      = "MQJMS/ASF/TopicLoad";
   // Other parameters
   private static int     poolSize   = 5;
   private static int     batchSize  = 10;
   // Reader for keyboard input
   private static BufferedReader kbd = new BufferedReader(
                                       new InputStreamReader( System.in ));

   /**************************************************************************
    *  main - Application entry point                                        *
    **************************************************************************/
   public static void main( String[] args ) throws JMSException, IOException
   {
      System.out.println("\nASFClient5 started\n");

      // Parse the command line
      parseArguments(args);

      TopicConnection     conn = null;
      MyServerSessionPool ssp  = null;

      // Obtain the two administered objects we need
      Object[] adminObjects = new Object[2];
      obtainAdminObjects( adminObjects );
      TopicConnectionFactory tcf = (TopicConnectionFactory)adminObjects[0];
      Topic t = (Topic)adminObjects[1];

      // Obtain a TopicConnection and start it
      conn = tcf.createTopicConnection();
      conn.start();

      // Reference to a TopicSession. Note this is only used for unsubscribing
      // durableConnectionConsumers, and is only created when we need it
      TopicSession sess = null;

      // Create a MessageListenerFactory to use
      CountingMessageListenerFactory cmlf = new CountingMessageListenerFactory();
      cmlf.setName("CountingFactory");

      // Create a ServerSessionPool to use
      ssp = new MyServerSessionPool( conn,         // The JMS Connection to use
                                     poolSize,           // The pool's capacity
                                     Session.AUTO_ACKNOWLEDGE,  // The ack mode
                                     cmlf );   // MessageListenerFactory to use

      // Display the options
      System.out.println("Please choose one of the following options:");
      System.out.println("   1. Create/reactivate a durable ConnectionConsumer");
      System.out.println("   2. Unsubscribe a durable ConnectionConsumer");
      System.out.println("   X. Exit\n");

      int action = 0;
      boolean retry = true;
      while( retry ) {
         System.out.print("Choice: ");

         // Get the user's input
         String response = kbd.readLine().toUpperCase();

         // Check the input and act accordingly
         if( response!=null ) {
            if( response.equals("1") ) {
               action = 1;
               retry=false;
            } else if( response.equals("2") ) {
               action = 2;
               retry=false;
            } else if( response.equals("X") ) {
               retry=false;
            }
         }
      }

      String name = null;
      if( action!=0 ) {
         // Obtain the durable consumer's name
         System.out.print("Subscriber's Name: ");
         name = kbd.readLine();
      }

      // Perform the user's request
      if( action==1 ) {
         try {
            // Create (or re-activate) the consumer
            conn.createDurableConnectionConsumer(t,name,null,ssp,batchSize);
            System.out.println("\nCreation completed OK. Printing statistics:\n");

         } catch( JMSException je ) {
            System.out.println("Error creating connection consumer: "+je );
         }

         // Now, every ten seconds for a minute, get the CountingMessageListener
         // factory to print out its stats
         for( int i=0; i<6; i++ ) {
            try {
               // Application thread sleeps for 10 seconds
               Thread.sleep(10000);
               System.out.println("   After "+(i+1)*10+" seconds:");
            } catch(InterruptedException ie) {
               // Ignore
            }
            cmlf.printStats();
         }

      } else if( action==2 ) {
         try {
            // Perform the session.unsubscribe
            sess = conn.createTopicSession(false,Session.AUTO_ACKNOWLEDGE);
            sess.unsubscribe(name);
            System.out.println("Unsubscribe completed OK");

         } catch( JMSException je ) {
            System.out.println("FAILED! Did you supply the correct name?");
         }
      }

      // Tidy up the state and exit
      if(conn!=null) conn.close();
      if(ssp!=null)  ssp.close();
      System.out.println("\nASFClient5 finished.");
   }

   /**************************************************************************
    *  parseArguments - Parse the given argument array, updating member      *
    *                   variables as we go. Terminates the application if    *
    *                   an error is encountered.                             *
    **************************************************************************/
   private static void parseArguments( String[] args ) {
      try {
         for( int i=0; i<args.length; i++ ) {
            String arg = args[i].toUpperCase();

            if( arg.equals("-NOJNDI"))         useJNDI   = false;
            else if( arg.equals("-URL"))       url       = args[++i];
            else if( arg.equals("-ICF"))       icf       = args[++i];
            else if( arg.equals("-TCFLOOKUP")) tcfLookup = args[++i];
            else if( arg.equals("-TLOOKUP"))   tLookup   = args[++i];
            else if( arg.equals("-T"))         tName     = args[++i];
            else if( arg.equals("-QMGR"))      qMgrName  = args[++i];
            else if( arg.equals("-POOLSIZE"))  poolSize  = Integer.parseInt(
                                                                  args[++i]);
            else if( arg.equals("-BATCHSIZE")) batchSize = Integer.parseInt(
                                                                  args[++i]);
            else
               throw new Exception("Unrecognised argument ("+arg+")");
         }
      } catch( Exception e ) {
         System.out.println("Error parsing arguments: "+e+"\nQuitting.");
         System.exit(-1);
      }
   }

   /**************************************************************************
    *  obtainAdminObjects - Obtains the two administered objects we need and *
    *                       places references to them in the given array.    *
    *                       The method used (JNDI versus explicit construct- *
    *                       ion) is determined by the value of 'useJNDI'.    *
    *                                                                        *
    *                       Terminates the application if any problems are   *
    *                       encountered.                                     *
    **************************************************************************/
   private static void obtainAdminObjects( Object[] outArray ) {
      try {
         if( useJNDI ) {
            // Set up JNDI environment parameters
            Hashtable env = new Hashtable();
            env.put( Context.PROVIDER_URL, url );
            env.put( Context.INITIAL_CONTEXT_FACTORY, icf );

            // Access the JNDI namespace
            Context ctx = new InitialContext(env);

            // Obtain the objects
            outArray[0] = ctx.lookup(tcfLookup);
            outArray[1] = ctx.lookup(tLookup);

         } else {
            // Create the objects explicitly - we need to refer to the
            // MQJMS-specific classes now
            com.ibm.mq.jms.MQTopicConnectionFactory tcf =
                           new com.ibm.mq.jms.MQTopicConnectionFactory();
            tcf.setQueueManager(qMgrName);
            tcf.setBrokerQueueManager(qMgrName);
            tcf.setClientId("ASFClient5");

            com.ibm.mq.jms.MQTopic t = new com.ibm.mq.jms.MQTopic();
            t.setBaseTopicName(tName);

            outArray[0] = tcf;
            outArray[1] = t;
         }
      } catch( Exception e ) {
         System.out.println("Error obtaining admin objects: "+e+"\nQuitting.");
         System.exit(-1);
      }
   }
}

